#include "mex.h"

inline bool locMax(double *in, int x, int y, int d, int width, int height) {
    double v = in[d*(height*width) + x*height + y];
    for(int dd=-1; dd<=1; dd++) {
        for(int yy=-1; yy<=1; yy++) {
            for(int xx=-1; xx<=1; xx++) {
                if(!(dd==0 && yy==0 && xx==0)) {
                    if(in[(d+dd)*(height*width) + (x+xx)*height + (y+yy)] >= v)
                        return false;
                }
            }
        }
    }
    return true;
}

inline bool locMin(double *in, int x, int y, int d, int width, int height) {
    double v = in[d*(height*width) + x*height + y];
    for(int dd=-1; dd<=1; dd++) {
        for(int yy=-1; yy<=1; yy++) {
            for(int xx=-1; xx<=1; xx++) {
                if(!(dd==0 && yy==0 && xx==0)) {
                    if(in[(d+dd)*(height*width) + (x+xx)*height + (y+yy)] <= v)
                        return false;
                }
            }
        }
    }
    return true;
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    // Funtion call: Max = localMax(D, )
    // D is a 3D matrix.
    
    int height, width, depth, ndim;
    const int *dims;
    double *in, *max;
    
    ndim = mxGetNumberOfDimensions(prhs[0]);
    if (ndim!=3) {
        mexErrMsgTxt("Input matrix must be 3D");
    }
    if (!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0])) {
        mexErrMsgTxt("First input must be a noncomplex matric of doubles.");
    }
    dims = mxGetDimensions(prhs[0]);
    height = dims[0];
    width = dims[1];
    depth = dims[2];
    if(depth<3) {
        mexErrMsgTxt("The depth need to be >= 3");
    }
    printf("%d x %d x %d\n", height, width, depth);
    
    in = mxGetPr(prhs[0]);

    plhs[0] = mxCreateNumericArray( ndim, dims, mxDOUBLE_CLASS, mxREAL);
    max = mxGetPr(plhs[0]);
    for(int d=1; d<depth-1; d++) {
        for(int y=1; y<height-1; y++) {
            for(int x=1;x<width-1; x++) {
                if((locMax(in, x, y, d, width, height) || locMin(in, x, y, d, width, height)) && in[d*(height*width) + x*height + y] > threshold)
                    max[d*(height*width) + x*height + y] = 1.0f;
                else
                    max[d*(height*width) + x*height + y] = 0.0f;
            }
        }
    }
                 
    
    /*
  sMap = mxGetPr(prhs[0]);
  maxR = (int)mxGetScalar(prhs[1]);
  threshold = mxGetScalar(prhs[2]);
   
  // Find the local maxima as ip
  ip = (double *)mxCalloc(0,sizeof(double));
  int nrIp = 0;
  for(int y=maxR; y<height-maxR; y++) {
    for(int x=maxR; x<width-maxR; x++) {
      int x0 = max(0, x-maxR);
      int x1 = min(width, x+maxR+1);
      int y0 = max(0, y-maxR);
      int y1 = min(height, y+maxR+1);
      double maxV=0;
      int maxX=width, maxY=height;
      for(int pX=x0; pX<x1; pX++) {
        for(int pY=y0; pY<y1; pY++) {
          int i=pX*height + pY; // Matlab uses row order
          if(sMap[i] > maxV) {
            maxV = sMap[i];
            maxX = pX;
            maxY = pY;
          }
        }
      }
      // Check if maximum is on (x,y)
      // TODO, check threshold
      if(maxX==x && maxY==y && maxV>threshold) {
        ip = addIP(ip, nrIp, maxX, maxY, maxV);
      }
    }
  }
   
  // Set the output pointer to the output matrix.
  plhs[0] = mxCreateDoubleMatrix(3, nrIp, mxREAL);
  mxSetPr(plhs[0], ip);
     */
}